body {
  font-family: Arial, Helvetica, sans-serif;
  background-color: #ccc;
  margin: 2em;
}

h1 {
  border-bottom: 3px solid;
}

input,
textarea {
  border: 0;
  margin-bottom: 1.5em;
}

input {
  height: 2em;
}

button {
  background-color: #f0f0f0;
  border-radius: 0.25em;
  height: 3em;
  width: 10em;
  font-size: 1em;
}

.errors {
  color: darkred;
}